import { Observable } from '@nativescript/core/data/observable';
import { ChangeType } from '@nativescript/core/data/observable-array';
const CHANGE = 'change';
class ObservableArrayInsideObservable extends Observable {
    constructor(parent) {
        super();
        this.parent = new WeakRef(parent);
        this._addArgs = {
            eventName: CHANGE,
            action: ChangeType.Add,
            index: null,
            removed: [],
            addedCount: 1,
        };
        this._deleteArgs = {
            eventName: CHANGE,
            action: ChangeType.Delete,
            index: null,
            removed: null,
            addedCount: 0,
        };
    }
    toString() {
        return '[ObservableArrayInsideObservable]';
    }
    notify(args) {
        const parent = this.parent && this.parent.get();
        args.object = parent || args.object;
        super.notify(args);
    }
    _notifyLengthChange() {
        const parent = this.parent && this.parent.get();
        if (parent) {
            const lengthChangedData = this._createPropertyChangeData('length', parent.length);
            this.notify(lengthChangedData);
        }
    }
}
let ID_INCREMENT = 0;
export class ObservableArray {
    constructor(...args) {
        this.shouldIgnoreSet = false;
        this.shouldIgnoreOps = false;
        let result;
        if (args.length === 1) {
            result = new Proxy(args[0].slice(), this);
        }
        else {
            result = new Proxy([...args], this);
        }
        this._observable = new ObservableArrayInsideObservable(result);
        result.id = ID_INCREMENT++;
        return result;
    }
    toString() {
        return '[ObservableArray]';
    }
    static [Symbol.hasInstance](instance) {
        return true;
    }
    getPrototypeOf(target) {
        return Observable.prototype;
    }
    get(target, prop) {
        if (typeof prop === 'symbol') {
            return target[prop];
        }
        const that = this;
        const notifier = that._observable;
        const val = target[prop];
        if (!val) {
            if (!val && typeof notifier[prop] === 'function') {
                return notifier[prop].bind(notifier);
            }
            else if (prop === 'getItem') {
                return function (index) {
                    return target[index];
                };
            }
            else if (prop === 'setItem') {
                return function (index, value) {
                    return this[index] = value;
                };
            }
        }
        else if (typeof val === 'function') {
            if (prop === 'toString') {
                return function () {
                    return JSON.stringify(target);
                };
            }
            if (this.shouldIgnoreOps) {
                return val;
            }
            if (prop === 'concat') {
                notifier._addArgs.index = this.length;
            }
            if (prop === 'reverse') {
                return function (el) {
                    that.shouldIgnoreSet = true;
                    const removed = [...target];
                    const result = Array.prototype[prop].apply(this, arguments);
                    that.shouldIgnoreSet = false;
                    notifier.notify({
                        eventName: CHANGE,
                        action: ChangeType.Splice,
                        index: 0,
                        removed,
                        addedCount: target.length,
                    });
                    return result;
                };
            }
            if (prop === 'push') {
                return function (el) {
                    notifier._addArgs.index = target.length;
                    that.shouldIgnoreSet = true;
                    const result = Array.prototype[prop].apply(this, arguments);
                    that.shouldIgnoreSet = false;
                    notifier._addArgs.addedCount = target.length - notifier._addArgs.index;
                    notifier.notify(notifier._addArgs);
                    notifier._notifyLengthChange();
                    return result;
                };
            }
            if (prop === 'unshift') {
                return function (el) {
                    const length = target.length;
                    that.shouldIgnoreSet = true;
                    const result = Array.prototype[prop].apply(this, arguments);
                    that.shouldIgnoreSet = false;
                    notifier._addArgs.index = 0;
                    notifier._addArgs.addedCount = result - length;
                    notifier.notify(notifier._addArgs);
                    notifier._notifyLengthChange();
                    return result;
                };
            }
            if (prop === 'pop') {
                return function (el) {
                    notifier._deleteArgs.index = target.length - 1;
                    that.shouldIgnoreSet = true;
                    const result = Array.prototype[prop].apply(this, arguments);
                    that.shouldIgnoreSet = false;
                    notifier._deleteArgs.removed = [result];
                    notifier.notify(notifier._deleteArgs);
                    notifier._notifyLengthChange();
                    return result;
                };
            }
            if (prop === 'shift') {
                return function (el) {
                    that.shouldIgnoreSet = true;
                    const result = Array.prototype[prop].apply(this, arguments);
                    that.shouldIgnoreSet = false;
                    notifier._deleteArgs.index = 0;
                    notifier._deleteArgs.removed = [result];
                    notifier.notify(notifier._deleteArgs);
                    notifier._notifyLengthChange();
                    return result;
                };
            }
            if (prop === 'splice') {
                return function (start) {
                    const length = target.length;
                    that.shouldIgnoreSet = true;
                    const result = Array.prototype[prop].apply(this, arguments);
                    that.shouldIgnoreSet = false;
                    notifier.notify({
                        eventName: CHANGE,
                        action: ChangeType.Splice,
                        index: Math.max(Math.min(start, length - (result.length > 0 ? 1 : 0)), 0),
                        removed: result,
                        addedCount: target.length + result.length - length,
                    });
                    if (target.length !== length) {
                        notifier._notifyLengthChange();
                    }
                    return result;
                };
            }
        }
        return val;
    }
    set(target, key, value) {
        let event;
        if (Number.isInteger(Number(key)) || key === 'length') {
            if (!this.shouldIgnoreSet) {
                event = {
                    eventName: CHANGE,
                    action: ChangeType.Update,
                    index: key,
                    removed: [target[key]],
                    addedCount: 1,
                };
            }
        }
        target[key] = value;
        if (event) {
            this._observable.notify(event);
        }
        return true;
    }
    deleteProperty(target, key) {
        const notifier = this._observable;
        let event;
        if (Number.isInteger(Number(key)) || key === 'length') {
            if (!this.shouldIgnoreSet) {
                event = {
                    eventName: CHANGE,
                    action: ChangeType.Update,
                    index: key,
                    removed: [target[key]],
                    addedCount: 1,
                };
            }
        }
        delete target[key];
        if (event) {
            notifier.notify(event);
        }
        return true;
    }
}
ObservableArray.changeEvent = CHANGE;
//# sourceMappingURL=observablearray.js.map